from .enums import GroupEventType
from .objects import BaseEventObject, group_event_objects
from .events_base import EventsBase
from pydantic import BaseModel
from typing import Optional


class BaseGroupEvent(BaseModel):
    type: Optional[GroupEventType] = None
    object: Optional[BaseEventObject] = None
    group_id: Optional[int] = None


class Message(BaseGroupEvent):
    object: "group_event_objects.MessageObject"


class MessageNew(BaseGroupEvent):
    object: "group_event_objects.MessageNewObject"


class MessageAllow(BaseGroupEvent):
    object: "group_event_objects.MessageAllowObject"


class MessageTypingState(BaseGroupEvent):
    object: "group_event_objects.MessageTypingStateObject"


class MessageDeny(BaseGroupEvent):
    object: "group_event_objects.MessageDenyObject"


class MessageEvent(BaseGroupEvent):
    object: "group_event_objects.MessageEventObject"


class PhotoNew(BaseGroupEvent):
    object: "group_event_objects.PhotosPhoto"


class PhotoComment(BaseGroupEvent):
    object: "group_event_objects.PhotoCommentObject"


class PhotoCommentDelete(BaseGroupEvent):
    object: "group_event_objects.PhotoCommentDeleteObject"


class AudioNew(BaseGroupEvent):
    object: "group_event_objects.AudioNewObject"


class VideoNew(BaseGroupEvent):
    object: "group_event_objects.VideoNewObject"


class VideoComment(BaseGroupEvent):
    object: "group_event_objects.VideoCommentObject"


class VideoCommentDelete(BaseGroupEvent):
    object: "group_event_objects.VideoCommentDeleteObject"


class WallPostNew(BaseGroupEvent):
    object: "group_event_objects.WallPostNewObject"


class WallReply(BaseGroupEvent):
    object: "group_event_objects.WallReplyNewObject"


class WallReplyDelete(BaseGroupEvent):
    object: "group_event_objects.WallReplyDeleteObject"


class Like(BaseGroupEvent):
    object: "group_event_objects.LikeObject"


class BoardPost(BaseGroupEvent):
    object: "group_event_objects.BoardPostNewObject"


class BoardPostDelete(BaseGroupEvent):
    object: "group_event_objects.BoardPostDeleteObject"


class MarketOrder(BaseGroupEvent):
    object: "group_event_objects.MarketOrderObject"


class MarketComment(BaseGroupEvent):
    object: "group_event_objects.MarketCommentNewObject"


class MarketCommentDelete(BaseGroupEvent):
    object: "group_event_objects.MarketCommentDeleteObject"


class GroupLeave(BaseGroupEvent):
    object: "group_event_objects.GroupLeaveObject"


class GroupJoin(BaseGroupEvent):
    object: "group_event_objects.GroupJoinObject"


class UserBlock(BaseGroupEvent):
    object: "group_event_objects.UserBlockObject"


class UserUnblock(BaseGroupEvent):
    object: "group_event_objects.UserUnblockObject"


class PollVoteNew(BaseGroupEvent):
    object: "group_event_objects.PollVoteNewObject"


class GroupOfficersEdit(BaseGroupEvent):
    object: "group_event_objects.GroupOfficersEditObject"


class GroupChangeSettings(BaseGroupEvent):
    object: "group_event_objects.GroupChangeSettingsObject"


class GroupChangePhoto(BaseGroupEvent):
    object: "group_event_objects.GroupChangePhotoObject"


class VkPayTransaction(BaseGroupEvent):
    object: "group_event_objects.VkPayTransactionObject"


class AppPayload(BaseGroupEvent):
    object: "group_event_objects.AppPayloadObject"


Message.update_forward_refs()
MessageNew.update_forward_refs()
MessageAllow.update_forward_refs()
MessageTypingState.update_forward_refs()
MessageDeny.update_forward_refs()
PhotoNew.update_forward_refs()
PhotoComment.update_forward_refs()
PhotoCommentDelete.update_forward_refs()
VideoComment.update_forward_refs()
VideoCommentDelete.update_forward_refs()
WallPostNew.update_forward_refs()
WallReply.update_forward_refs()
WallReplyDelete.update_forward_refs()
BoardPost.update_forward_refs()
BoardPostDelete.update_forward_refs()
MarketComment.update_forward_refs()
MarketCommentDelete.update_forward_refs()
GroupLeave.update_forward_refs()
GroupJoin.update_forward_refs()
UserBlock.update_forward_refs()
UserUnblock.update_forward_refs()
PollVoteNew.update_forward_refs()
GroupOfficersEdit.update_forward_refs()
GroupChangeSettings.update_forward_refs()
GroupChangePhoto.update_forward_refs()
VkPayTransaction.update_forward_refs()
AppPayload.update_forward_refs()

DEFAULT_EVENTS_BASE_GROUP = EventsBase(GroupEventType)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MESSAGE_NEW, MessageNew)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MESSAGE_REPLY, Message)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MESSAGE_EDIT, Message)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MESSAGE_ALLOW, MessageAllow)
DEFAULT_EVENTS_BASE_GROUP.register(
    GroupEventType.MESSAGE_TYPING_STATE, MessageTypingState
)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MESSAGE_DENY, MessageDeny)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MESSAGE_EVENT, MessageEvent)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.PHOTO_NEW, PhotoNew)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.PHOTO_COMMENT_NEW, PhotoComment)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.PHOTO_COMMENT_EDIT, PhotoComment)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.PHOTO_COMMENT_RESTORE, PhotoComment)
DEFAULT_EVENTS_BASE_GROUP.register(
    GroupEventType.PHOTO_COMMENT_DELETE, PhotoCommentDelete
)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.AUDIO_NEW, AudioNew)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.VIDEO_NEW, VideoNew)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.VIDEO_COMMENT_NEW, VideoComment)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.VIDEO_COMMENT_EDIT, VideoComment)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.VIDEO_COMMENT_RESTORE, VideoComment)
DEFAULT_EVENTS_BASE_GROUP.register(
    GroupEventType.VIDEO_COMMENT_DELETE, VideoCommentDelete
)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.WALL_POST_NEW, WallPostNew)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.WALL_REPOST, WallPostNew)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.WALL_REPLY_NEW, WallReply)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.WALL_REPLY_EDIT, WallReply)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.WALL_REPLY_RESTORE, WallReply)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.WALL_REPLY_DELETE, WallReplyDelete)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.LIKE_ADD, Like)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.LIKE_REMOVE, Like)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.BOARD_POST_NEW, BoardPost)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.BOARD_POST_EDIT, BoardPost)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.BOARD_POST_RESTORE, BoardPost)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.BOARD_POST_DELETE, BoardPostDelete)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MARKET_ORDER_NEW, MarketOrder)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MARKET_ORDER_EDIT, MarketOrder)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.BOARD_POST_DELETE, BoardPostDelete)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MARKET_COMMENT_NEW, MarketComment)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MARKET_COMMENT_EDIT, MarketComment)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.MARKET_COMMENT_RESTORE, MarketComment)
DEFAULT_EVENTS_BASE_GROUP.register(
    GroupEventType.MARKET_COMMENT_DELETE, MarketCommentDelete
)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.GROUP_LEAVE, GroupLeave)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.GROUP_JOIN, GroupJoin)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.USER_BLOCK, UserBlock)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.USER_UNBLOCK, UserUnblock)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.POLL_VOTE_NEW, PollVoteNew)
DEFAULT_EVENTS_BASE_GROUP.register(
    GroupEventType.GROUP_OFFICERS_EDIT, GroupOfficersEdit
)
DEFAULT_EVENTS_BASE_GROUP.register(
    GroupEventType.GROUP_CHANGE_SETTINGS, GroupChangeSettings
)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.GROUP_CHANGE_PHOTO, GroupChangePhoto)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.VKPAY_TRANSACTION, VkPayTransaction)
DEFAULT_EVENTS_BASE_GROUP.register(GroupEventType.APP_PAYLOAD, AppPayload)

__all__ = (
    "DEFAULT_EVENTS_BASE_GROUP",
    "BaseGroupEvent",
    "MessageNew",
    "Message",
    "MessageAllow",
    "MessageTypingState",
    "MessageDeny",
    "MessageEvent",
    "PhotoNew",
    "PhotoComment",
    "PhotoCommentDelete",
    "AudioNew",
    "VideoNew",
    "VideoComment",
    "VideoCommentDelete",
    "WallPostNew",
    "WallReply",
    "WallReplyDelete",
    "Like",
    "BoardPost",
    "BoardPostDelete",
    "MarketComment",
    "MarketCommentDelete",
    "GroupJoin",
    "GroupLeave",
    "UserBlock",
    "UserUnblock",
    "PollVoteNew",
    "GroupOfficersEdit",
    "GroupChangeSettings",
    "GroupChangePhoto",
    "VkPayTransaction",
    "AppPayload",
)



from .base_event_object import BaseEventObject
from typing import Optional, Union, List


class MessageFieldsObject(BaseEventObject):
    message_id: Optional[int] = None
    peer_id: Optional[int] = None
    timestamp: Optional[int] = None
    text: Optional[str] = None
    subject: Optional[str] = None
    info: Optional[str] = None
    attachments: Optional[Union[dict, list]] = None
    random_id: Optional[int] = None


class ReplaceMessageFlagsObject(MessageFieldsObject):
    flags: Optional[int] = None


class InstallMessageFlagsObject(MessageFieldsObject):
    mask: Optional[int] = None


class ResetMessageFlagsObject(MessageFieldsObject):
    mask: Optional[int] = None


class EditMessageObject(BaseEventObject):
    message_id: Optional[int] = None
    mask: Optional[int] = None
    peer_id: Optional[int] = None
    timestamp: Optional[int] = None
    new_text: Optional[str] = ""
    attachments: Optional[Union[dict, list]] = None


class InReadObject(BaseEventObject):
    peer_id: Optional[int] = None
    local_id: Optional[int] = None


class OutReadObject(InReadObject):
    pass


class FriendOnlineObject(BaseEventObject):
    user_id: Optional[int] = None
    extra: Optional[int] = 0
    timestamp: Optional[int] = None


class FriendOfflineObject(BaseEventObject):
    user_id: Optional[int] = None
    flags: Optional[int] = 0
    timestamp: Optional[int] = None


class ResetDialogFlagsObject(BaseEventObject):
    peer_id: Optional[int] = None
    mask: Optional[int] = None


class ReplaceDialogFlagsObject(BaseEventObject):
    peer_id: Optional[int] = None
    flags: Optional[int] = None


class InstallDialogFlagsObject(ResetDialogFlagsObject):
    pass


class DeleteMessagesObject(InReadObject):
    pass


class ChangeConversationParamsObject(BaseEventObject):
    chat_id: Optional[int] = None
    self: Optional[int] = None


class DialogTypingStateObject(BaseEventObject):
    user_id: Optional[int] = None
    flags: Optional[int] = None


class ConversationTypingStateObject(BaseEventObject):
    user_id: Optional[int] = None
    chat_id: Optional[int] = None


class CallObject(BaseEventObject):
    user_id: Optional[int] = None
    call_id: Optional[int] = None


class CounterObject(BaseEventObject):
    count: Optional[int] = None


class ChangedNotificationsSettingsObject(BaseEventObject):
    peer_id: Optional[int] = None
    sound: Optional[int] = None
    disabled_until: Optional[int] = None


class RestoreDeletedObject(DeleteMessagesObject):
    pass


class ChatInfoEditObject(BaseEventObject):
    type_id: Optional[int] = None
    peer_id: Optional[int] = None
    info: Optional[Union[str, int]] = None


class ChatVoiceMessageStatesObject(BaseEventObject):
    user_ids: Optional[List[int]] = None
    peer_id: Optional[int] = None
    total_count: Optional[int] = None
    ts: Optional[int] = None


class ChatEditObject(BaseEventObject):
    chat_id: Optional[int] = None
    self: Optional[int] = None


MessageFieldsObject.update_forward_refs()
ReplaceMessageFlagsObject.update_forward_refs()
InstallMessageFlagsObject.update_forward_refs()
ResetMessageFlagsObject.update_forward_refs()
EditMessageObject.update_forward_refs()
InReadObject.update_forward_refs()
OutReadObject.update_forward_refs()
FriendOnlineObject.update_forward_refs()
FriendOfflineObject.update_forward_refs()
ResetDialogFlagsObject.update_forward_refs()
ReplaceDialogFlagsObject.update_forward_refs()
InstallDialogFlagsObject.update_forward_refs()
DeleteMessagesObject.update_forward_refs()
ChangeConversationParamsObject.update_forward_refs()
DialogTypingStateObject.update_forward_refs()
ConversationTypingStateObject.update_forward_refs()
CallObject.update_forward_refs()
CounterObject.update_forward_refs()
ChangedNotificationsSettingsObject.update_forward_refs()
RestoreDeletedObject.update_forward_refs()
ChatInfoEditObject.update_forward_refs()
ChatVoiceMessageStatesObject.update_forward_refs()
ChatEditObject.update_forward_refs()



import typing
from typing import Optional, List

from vkbottle_types.responses import messages, base
from .base_category import BaseCategory

if typing.TYPE_CHECKING:
    from vkbottle_types.objects import users as objects_users, base as objects_base


class MessagesCategory(BaseCategory):
    async def add_chat_user(
        self,
        chat_id: int,
        user_id: Optional[int] = None,
        visible_messages_count: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Adds a new user to a chat.
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be added to the chat.
        :param visible_messages_count:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.addChatUser", params)
        ).response

    async def allow_messages_from_group(
        self, group_id: int, key: Optional[str] = None, **kwargs
    ) -> base.OkResponseModel:
        """Allows sending messages from community to the current user.
        :param group_id: Group ID.
        :param key:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.allowMessagesFromGroup", params)
        ).response

    async def create_chat(
        self,
        user_ids: Optional[List[int]] = None,
        title: Optional[str] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.CreateChatResponseModel:
        """Creates a chat with several participants.
        :param user_ids: IDs of the users to be added to the chat.
        :param title: Chat title.
        :param group_id:
        """

        params = self.get_set_params(locals())
        return messages.CreateChatResponse(
            **await self.api.request("messages.createChat", params)
        ).response

    async def delete(
        self,
        message_ids: Optional[List[int]] = None,
        spam: Optional[bool] = None,
        group_id: Optional[int] = None,
        delete_for_all: Optional[bool] = None,
        **kwargs
    ) -> messages.DeleteResponseModel:
        """Deletes one or more messages.
        :param message_ids: Message IDs.
        :param spam: '1' — to mark message as spam.
        :param group_id: Group ID (for group messages with user access token)
        :param delete_for_all: '1' — delete message for for all.
        """

        params = self.get_set_params(locals())
        return messages.DeleteResponse(
            **await self.api.request("messages.delete", params)
        ).response

    async def delete_chat_photo(
        self, chat_id: int, group_id: Optional[int] = None, **kwargs
    ) -> messages.DeleteChatPhotoResponseModel:
        """Deletes a chat's cover picture.
        :param chat_id: Chat ID.
        :param group_id:
        """

        params = self.get_set_params(locals())
        return messages.DeleteChatPhotoResponse(
            **await self.api.request("messages.deleteChatPhoto", params)
        ).response

    async def delete_conversation(
        self,
        user_id: Optional[int] = None,
        peer_id: Optional[int] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.DeleteConversationResponseModel:
        """Deletes all private messages in a conversation.
        :param user_id: User ID. To clear a chat history use 'chat_id'
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with user access token)
        """

        params = self.get_set_params(locals())
        return messages.DeleteConversationResponse(
            **await self.api.request("messages.deleteConversation", params)
        ).response

    async def deny_messages_from_group(
        self, group_id: int, **kwargs
    ) -> base.OkResponseModel:
        """Denies sending message from community to the current user.
        :param group_id: Group ID.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.denyMessagesFromGroup", params)
        ).response

    async def edit(
        self,
        peer_id: int,
        message: Optional[str] = None,
        lat: Optional[float] = None,
        long: Optional[float] = None,
        attachment: Optional[str] = None,
        keep_forward_messages: Optional[bool] = None,
        keep_snippets: Optional[bool] = None,
        group_id: Optional[int] = None,
        dont_parse_links: Optional[bool] = None,
        message_id: Optional[int] = None,
        conversation_message_id: Optional[int] = None,
        template: Optional[str] = None,
        keyboard: Optional[str] = None,
        **kwargs
    ) -> messages.EditResponseModel:
        """Edits the message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param message: (Required if 'attachments' is not set., **kwargs) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set., **kwargs) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param keep_forward_messages: '1' — to keep forwarded, messages.
        :param keep_snippets: '1' — to keep attached snippets.
        :param group_id: Group ID (for group messages with user access token)
        :param dont_parse_links:
        :param message_id:
        :param conversation_message_id:
        :param template:
        :param keyboard:
        """

        params = self.get_set_params(locals())
        return messages.EditResponse(
            **await self.api.request("messages.edit", params)
        ).response

    async def edit_chat(
        self, chat_id: int, title: Optional[str] = None, **kwargs
    ) -> base.OkResponseModel:
        """Edits the title of a chat.
        :param chat_id: Chat ID.
        :param title: New title of the chat.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.editChat", params)
        ).response

    async def get_by_conversation_message_id(
        self,
        peer_id: int,
        conversation_message_ids: List[int],
        extended: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetByConversationMessageIdResponseModel:
        """Returns messages by their IDs within the conversation.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param conversation_message_ids: Conversation message IDs.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.GetByConversationMessageIdResponse(
            **await self.api.request("messages.getByConversationMessageId", params)
        ).response

    async def get_by_id(
        self,
        message_ids: List[int],
        preview_length: Optional[int] = None,
        extended: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetByIdExtendedResponseModel:
        """Returns messages by their IDs.
        :param message_ids: Message IDs.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.GetByIdExtendedResponse(
            **await self.api.request("messages.getById", params)
        ).response

    async def get_chat_preview(
        self,
        peer_id: Optional[int] = None,
        link: Optional[str] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        **kwargs
    ) -> messages.GetChatPreviewResponseModel:
        """messages.getChatPreview method
        :param peer_id:
        :param link: Invitation link.
        :param fields: Profile fields to return.
        """

        params = self.get_set_params(locals())
        return messages.GetChatPreviewResponse(
            **await self.api.request("messages.getChatPreview", params)
        ).response

    async def get_conversation_members(
        self,
        peer_id: int,
        fields: Optional[List["objects_users.Fields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetConversationMembersResponseModel:
        """Returns a list of IDs of users participating in a chat.
        :param peer_id: Peer ID.
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.GetConversationMembersResponse(
            **await self.api.request("messages.getConversationMembers", params)
        ).response

    async def get_conversations(
        self,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        filter: Optional[str] = None,
        extended: Optional[bool] = None,
        start_message_id: Optional[int] = None,
        fields: Optional[List["objects_base.UserGroupFields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetConversationsResponseModel:
        """Returns a list of the current user's conversations.
        :param offset: Offset needed to return a specific subset of conversations.
        :param count: Number of conversations to return.
        :param filter: Filter to apply: 'all' — all conversations, 'unread' — conversations with unread messages, 'important' — conversations, marked as important (only for community messages), 'unanswered' — conversations, marked as unanswered (only for community messages)
        :param extended: '1' — return extra information about users and communities
        :param start_message_id: ID of the message from what to return dialogs.
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.GetConversationsResponse(
            **await self.api.request("messages.getConversations", params)
        ).response

    async def get_conversations_by_id(
        self,
        peer_ids: List[int],
        extended: Optional[bool] = None,
        fields: Optional[List["objects_base.UserGroupFields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetConversationsByIdExtendedResponseModel:
        """Returns conversations by their IDs
        :param peer_ids: Destination IDs. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param extended: Return extended properties
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.GetConversationsByIdExtendedResponse(
            **await self.api.request("messages.getConversationsById", params)
        ).response

    async def get_history(
        self,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        user_id: Optional[int] = None,
        peer_id: Optional[int] = None,
        start_message_id: Optional[int] = None,
        rev: Optional[int] = None,
        extended: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetHistoryResponseModel:
        """Returns message history for the specified user or group chat.
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param user_id: ID of the user whose message history you want to return.
        :param peer_id:
        :param start_message_id: Starting message ID from which to return history.
        :param rev: Sort order: '1' — return messages in chronological order. '0' — return messages in reverse chronological order.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.GetHistoryResponse(
            **await self.api.request("messages.getHistory", params)
        ).response

    async def get_history_attachments(
        self,
        peer_id: int,
        media_type: Optional[str] = None,
        start_from: Optional[str] = None,
        count: Optional[int] = None,
        photo_sizes: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        group_id: Optional[int] = None,
        preserve_order: Optional[bool] = None,
        max_forwards_level: Optional[int] = None,
        **kwargs
    ) -> messages.GetHistoryAttachmentsResponseModel:
        """Returns media files from the dialog or group chat.
        :param peer_id: Peer ID. ", For group chat: '2000000000 + chat ID' , , For community: '-community ID'"
        :param media_type: Type of media files to return: *'photo',, *'video',, *'audio',, *'doc',, *'link'.,*'market'.,*'wall'.,*'share'
        :param start_from: Message ID to start return results from.
        :param count: Number of objects to return.
        :param photo_sizes: '1' — to return photo sizes in a
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param group_id: Group ID (for group messages with group access token)
        :param preserve_order:
        :param max_forwards_level:
        """

        params = self.get_set_params(locals())
        return messages.GetHistoryAttachmentsResponse(
            **await self.api.request("messages.getHistoryAttachments", params)
        ).response

    async def get_invite_link(
        self,
        peer_id: int,
        reset: Optional[bool] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.GetInviteLinkResponseModel:
        """messages.getInviteLink method
        :param peer_id: Destination ID.
        :param reset: 1 — to generate new link (revoke previous), 0 — to return previous link.
        :param group_id: Group ID
        """

        params = self.get_set_params(locals())
        return messages.GetInviteLinkResponse(
            **await self.api.request("messages.getInviteLink", params)
        ).response

    async def get_last_activity(
        self, user_id: int, **kwargs
    ) -> messages.GetLastActivityResponseModel:
        """Returns a user's current status and date of last activity.
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        return messages.GetLastActivityResponse(
            **await self.api.request("messages.getLastActivity", params)
        ).response

    async def get_long_poll_history(
        self,
        ts: Optional[int] = None,
        pts: Optional[int] = None,
        preview_length: Optional[int] = None,
        onlines: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        events_limit: Optional[int] = None,
        msgs_limit: Optional[int] = None,
        max_msg_id: Optional[int] = None,
        group_id: Optional[int] = None,
        lp_version: Optional[int] = None,
        last_n: Optional[int] = None,
        credentials: Optional[bool] = None,
        **kwargs
    ) -> messages.GetLongPollHistoryResponseModel:
        """Returns updates in user's private messages.
        :param ts: Last value of the 'ts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param pts: Lsat value of 'pts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param onlines: '1' — to return history with online users only.
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param events_limit: Maximum number of events to return.
        :param msgs_limit: Maximum number of messages to return.
        :param max_msg_id: Maximum ID of the message among existing ones in the local copy. Both messages received with API methods (for example, , ), and data received from a Long Poll server (events with code 4, **kwargs) are taken into account.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version:
        :param last_n:
        :param credentials:
        """

        params = self.get_set_params(locals())
        return messages.GetLongPollHistoryResponse(
            **await self.api.request("messages.getLongPollHistory", params)
        ).response

    async def get_long_poll_server(
        self,
        need_pts: Optional[bool] = None,
        group_id: Optional[int] = None,
        lp_version: Optional[int] = None,
        **kwargs
    ) -> messages.GetLongPollServerResponseModel:
        """Returns data required for connection to a Long Poll server.
        :param need_pts: '1' — to return the 'pts' field, needed for the [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version: Long poll version
        """

        params = self.get_set_params(locals())
        return messages.GetLongPollServerResponse(
            **await self.api.request("messages.getLongPollServer", params)
        ).response

    async def is_messages_from_group_allowed(
        self, group_id: int, user_id: int, **kwargs
    ) -> messages.IsMessagesFromGroupAllowedResponseModel:
        """Returns information whether sending messages from the community to current user is allowed.
        :param group_id: Group ID.
        :param user_id: User ID.
        """

        params = self.get_set_params(locals())
        return messages.IsMessagesFromGroupAllowedResponse(
            **await self.api.request("messages.isMessagesFromGroupAllowed", params)
        ).response

    async def join_chat_by_invite_link(
        self, link: str, **kwargs
    ) -> messages.JoinChatByInviteLinkResponseModel:
        """messages.joinChatByInviteLink method
        :param link: Invitation link.
        """

        params = self.get_set_params(locals())
        return messages.JoinChatByInviteLinkResponse(
            **await self.api.request("messages.joinChatByInviteLink", params)
        ).response

    async def mark_as_answered_conversation(
        self,
        peer_id: int,
        answered: Optional[bool] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Marks and unmarks conversations as unanswered.
        :param peer_id: ID of conversation to mark as important.
        :param answered: '1' — to mark as answered, '0' — to remove the mark
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.markAsAnsweredConversation", params)
        ).response

    async def mark_as_important(
        self,
        message_ids: Optional[List[int]] = None,
        important: Optional[int] = None,
        **kwargs
    ) -> messages.MarkAsImportantResponseModel:
        """Marks and unmarks messages as important (starred).
        :param message_ids: IDs of messages to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star
        """

        params = self.get_set_params(locals())
        return messages.MarkAsImportantResponse(
            **await self.api.request("messages.markAsImportant", params)
        ).response

    async def mark_as_important_conversation(
        self,
        peer_id: int,
        important: Optional[bool] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Marks and unmarks conversations as important.
        :param peer_id: ID of conversation to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.markAsImportantConversation", params)
        ).response

    async def mark_as_read(
        self,
        message_ids: Optional[List[int]] = None,
        peer_id: Optional[int] = None,
        start_message_id: Optional[int] = None,
        group_id: Optional[int] = None,
        mark_conversation_as_read: Optional[bool] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Marks messages as read.
        :param message_ids: IDs of messages to mark as read.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param start_message_id: Message ID to start from.
        :param group_id: Group ID (for group messages with user access token)
        :param mark_conversation_as_read:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.markAsRead", params)
        ).response

    async def pin(
        self, peer_id: int, message_id: Optional[int] = None, **kwargs
    ) -> messages.PinResponseModel:
        """Pin a message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param message_id:
        """

        params = self.get_set_params(locals())
        return messages.PinResponse(
            **await self.api.request("messages.pin", params)
        ).response

    async def remove_chat_user(
        self,
        chat_id: int,
        user_id: Optional[int] = None,
        member_id: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Allows the current user to leave a chat or, if the current user started the chat, allows the user to remove another user from the chat.
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be removed from the chat.
        :param member_id:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.removeChatUser", params)
        ).response

    async def restore(
        self, message_id: int, group_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Restores a deleted message.
        :param message_id: ID of a previously-deleted message to restore.
        :param group_id: Group ID (for group messages with user access token)
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.restore", params)
        ).response

    async def search(
        self,
        q: Optional[str] = None,
        peer_id: Optional[int] = None,
        date: Optional[int] = None,
        preview_length: Optional[int] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        extended: Optional[bool] = None,
        fields: Optional[List[str]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.SearchResponseModel:
        """Returns a list of the current user's private messages that match search criteria.
        :param q: Search query string.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param date: Date to search message before in Unixtime.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param extended:
        :param fields:
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return messages.SearchResponse(
            **await self.api.request("messages.search", params)
        ).response

    async def search_conversations(
        self,
        q: Optional[str] = None,
        count: Optional[int] = None,
        extended: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> messages.SearchConversationsResponseModel:
        """Returns a list of the current user's conversations that match search criteria.
        :param q: Search query string.
        :param count: Maximum number of results.
        :param extended: '1' — return extra information about users and communities
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with user access token)
        """

        params = self.get_set_params(locals())
        return messages.SearchConversationsResponse(
            **await self.api.request("messages.searchConversations", params)
        ).response

    async def send(
        self,
        user_id: Optional[int] = None,
        random_id: Optional[int] = None,
        peer_id: Optional[int] = None,
        domain: Optional[str] = None,
        chat_id: Optional[int] = None,
        user_ids: Optional[List[int]] = None,
        message: Optional[str] = None,
        lat: Optional[float] = None,
        long: Optional[float] = None,
        attachment: Optional[str] = None,
        reply_to: Optional[int] = None,
        forward_messages: Optional[List[int]] = None,
        sticker_id: Optional[int] = None,
        group_id: Optional[int] = None,
        keyboard: Optional[str] = None,
        payload: Optional[str] = None,
        dont_parse_links: Optional[bool] = None,
        disable_mentions: Optional[bool] = None,
        intent: Optional[str] = None,
        subscribe_id: Optional[int] = None,
        **kwargs
    ) -> messages.SendResponseModel:
        """Sends a message.
        :param user_id: User ID (by default — current user).
        :param random_id: Unique identifier to avoid resending the message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param domain: User's short address (for example, 'illarionov').
        :param chat_id: ID of conversation the message will relate to.
        :param user_ids: IDs of message recipients (if new conversation shall be started).
        :param message: (Required if 'attachments' is not set., **kwargs) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set., **kwargs) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param reply_to:
        :param forward_messages: ID of forwarded messages, separated with a comma. Listed messages of the sender will be shown in the message body at the recipient's. Example: "123,431,544"
        :param sticker_id: Sticker id.
        :param group_id: Group ID (for group messages with group access token)
        :param keyboard:
        :param payload:
        :param dont_parse_links:
        :param disable_mentions:
        :param intent:
        :param subscribe_id:
        """

        params = self.get_set_params(locals())
        return messages.SendResponse(
            **await self.api.request("messages.send", params)
        ).response

    async def send_message_event_answer(
        self,
        event_id: str,
        user_id: int,
        peer_id: int,
        event_data: Optional[str] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """messages.sendMessageEventAnswer method
        :param event_id:
        :param user_id:
        :param peer_id:
        :param event_data:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.sendMessageEventAnswer", params)
        ).response

    async def set_activity(
        self,
        user_id: Optional[int] = None,
        type: Optional[str] = None,
        peer_id: Optional[int] = None,
        group_id: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Changes the status of a user as typing in a conversation.
        :param user_id: User ID.
        :param type: 'typing' — user has started to type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with group access token)
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.setActivity", params)
        ).response

    async def set_chat_photo(
        self, file: str, **kwargs
    ) -> messages.SetChatPhotoResponseModel:
        """Sets a previously-uploaded picture as the cover picture of a chat.
        :param file: Upload URL from the 'response' field returned by the [vk.com/dev/photos.getChatUploadServer|photos.getChatUploadServer] method upon successfully uploading an image.
        """

        params = self.get_set_params(locals())
        return messages.SetChatPhotoResponse(
            **await self.api.request("messages.setChatPhoto", params)
        ).response

    async def unpin(
        self, peer_id: int, group_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """messages.unpin method
        :param peer_id:
        :param group_id:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("messages.unpin", params)
        ).response



import typing
from typing import Optional, List

from vkbottle_types.responses import photos, base
from .base_category import BaseCategory

if typing.TYPE_CHECKING:
    from vkbottle_types.objects import users as objects_users


class PhotosCategory(BaseCategory):
    async def confirm_tag(
        self, photo_id: str, tag_id: int, owner_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Confirms a tag on a photo.
        :param photo_id: Photo ID.
        :param tag_id: Tag ID.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.confirmTag", params)
        ).response

    async def copy(
        self, owner_id: int, photo_id: int, access_key: Optional[str] = None, **kwargs
    ) -> photos.CopyResponseModel:
        """Allows to copy a photo to the "Saved photos" album
        :param owner_id: photo's owner ID
        :param photo_id: photo ID
        :param access_key: for private photos
        """

        params = self.get_set_params(locals())
        return photos.CopyResponse(
            **await self.api.request("photos.copy", params)
        ).response

    async def create_album(
        self,
        title: str,
        group_id: Optional[int] = None,
        description: Optional[str] = None,
        privacy_view: Optional[List[str]] = None,
        privacy_comment: Optional[List[str]] = None,
        upload_by_admins_only: Optional[bool] = None,
        comments_disabled: Optional[bool] = None,
        **kwargs
    ) -> photos.CreateAlbumResponseModel:
        """Creates an empty photo album.
        :param title: Album title.
        :param group_id: ID of the community in which the album will be created.
        :param description: Album description.
        :param privacy_view:
        :param privacy_comment:
        :param upload_by_admins_only:
        :param comments_disabled:
        """

        params = self.get_set_params(locals())
        return photos.CreateAlbumResponse(
            **await self.api.request("photos.createAlbum", params)
        ).response

    async def create_comment(
        self,
        photo_id: int,
        owner_id: Optional[int] = None,
        message: Optional[str] = None,
        attachments: Optional[List[str]] = None,
        from_group: Optional[bool] = None,
        reply_to_comment: Optional[int] = None,
        sticker_id: Optional[int] = None,
        access_key: Optional[str] = None,
        guid: Optional[str] = None,
        **kwargs
    ) -> photos.CreateCommentResponseModel:
        """Adds a new comment on the photo.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param message: Comment text.
        :param attachments: (Required if 'message' is not set., **kwargs) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — Media attachment owner ID. '<media_id>' — Media attachment ID. Example: "photo100172_166443618,photo66748_265827614"
        :param from_group: '1' — to post a comment from the community
        :param reply_to_comment:
        :param sticker_id:
        :param access_key:
        :param guid:
        """

        params = self.get_set_params(locals())
        return photos.CreateCommentResponse(
            **await self.api.request("photos.createComment", params)
        ).response

    async def delete(
        self, photo_id: int, owner_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Deletes a photo.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.delete", params)
        ).response

    async def delete_album(
        self, album_id: int, group_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Deletes a photo album belonging to the current user.
        :param album_id: Album ID.
        :param group_id: ID of the community that owns the album.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.deleteAlbum", params)
        ).response

    async def delete_comment(
        self, comment_id: int, owner_id: Optional[int] = None, **kwargs
    ) -> photos.DeleteCommentResponseModel:
        """Deletes a comment on the photo.
        :param comment_id: Comment ID.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return photos.DeleteCommentResponse(
            **await self.api.request("photos.deleteComment", params)
        ).response

    async def edit(
        self,
        photo_id: int,
        owner_id: Optional[int] = None,
        caption: Optional[str] = None,
        latitude: Optional[float] = None,
        longitude: Optional[float] = None,
        place_str: Optional[str] = None,
        foursquare_id: Optional[str] = None,
        delete_place: Optional[bool] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Edits the caption of a photo.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param caption: New caption for the photo. If this parameter is not set, it is considered to be equal to an empty string.
        :param latitude:
        :param longitude:
        :param place_str:
        :param foursquare_id:
        :param delete_place:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(**await self.api.request("photos.edit", params)).response

    async def edit_album(
        self,
        album_id: int,
        title: Optional[str] = None,
        description: Optional[str] = None,
        owner_id: Optional[int] = None,
        privacy_view: Optional[List[str]] = None,
        privacy_comment: Optional[List[str]] = None,
        upload_by_admins_only: Optional[bool] = None,
        comments_disabled: Optional[bool] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Edits information about a photo album.
        :param album_id: ID of the photo album to be edited.
        :param title: New album title.
        :param description: New album description.
        :param owner_id: ID of the user or community that owns the album.
        :param privacy_view:
        :param privacy_comment:
        :param upload_by_admins_only:
        :param comments_disabled:
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.editAlbum", params)
        ).response

    async def edit_comment(
        self,
        comment_id: int,
        owner_id: Optional[int] = None,
        message: Optional[str] = None,
        attachments: Optional[List[str]] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Edits a comment on a photo.
        :param comment_id: Comment ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param message: New text of the comment.
        :param attachments: (Required if 'message' is not set., **kwargs) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — Media attachment owner ID. '<media_id>' — Media attachment ID. Example: "photo100172_166443618,photo66748_265827614"
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.editComment", params)
        ).response

    async def get(
        self,
        owner_id: Optional[int] = None,
        album_id: Optional[str] = None,
        photo_ids: Optional[List[str]] = None,
        rev: Optional[bool] = None,
        extended: Optional[bool] = None,
        feed_type: Optional[str] = None,
        feed: Optional[int] = None,
        photo_sizes: Optional[bool] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        **kwargs
    ) -> photos.GetExtendedResponseModel:
        """Returns a list of a user's or community's photos.
        :param owner_id: ID of the user or community that owns the photos. Use a negative value to designate a community ID.
        :param album_id: Photo album ID. To return information about photos from service albums, use the following string values: 'profile, wall, saved'.
        :param photo_ids: Photo IDs.
        :param rev: Sort order: '1' — reverse chronological, '0' — chronological
        :param extended: '1' — to return additional 'likes', 'comments', and 'tags' fields, '0' — (default)
        :param feed_type: Type of feed obtained in 'feed' field of the method.
        :param feed: unixtime, that can be obtained with [vk.com/dev/newsfeed.get|newsfeed.get] method in date field to get all photos uploaded by the user on a specific day, or photos the user has been tagged on. Also, 'uid' parameter of the user the event happened with shall be specified.
        :param photo_sizes: '1' — to return photo sizes in a [vk.com/dev/photo_sizes|special format]
        :param offset:
        :param count:
        """

        params = self.get_set_params(locals())
        return photos.GetExtendedResponse(
            **await self.api.request("photos.get", params)
        ).response

    async def get_albums(
        self,
        owner_id: Optional[int] = None,
        album_ids: Optional[List[int]] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        need_system: Optional[bool] = None,
        need_covers: Optional[bool] = None,
        photo_sizes: Optional[bool] = None,
        **kwargs
    ) -> photos.GetAlbumsResponseModel:
        """Returns a list of a user's or community's photo albums.
        :param owner_id: ID of the user or community that owns the albums.
        :param album_ids: Album IDs.
        :param offset: Offset needed to return a specific subset of albums.
        :param count: Number of albums to return.
        :param need_system: '1' — to return system albums with negative IDs
        :param need_covers: '1' — to return an additional 'thumb_src' field, '0' — (default)
        :param photo_sizes: '1' — to return photo sizes in a
        """

        params = self.get_set_params(locals())
        return photos.GetAlbumsResponse(
            **await self.api.request("photos.getAlbums", params)
        ).response

    async def get_albums_count(
        self, user_id: Optional[int] = None, group_id: Optional[int] = None, **kwargs
    ) -> photos.GetAlbumsCountResponseModel:
        """Returns the number of photo albums belonging to a user or community.
        :param user_id: User ID.
        :param group_id: Community ID.
        """

        params = self.get_set_params(locals())
        return photos.GetAlbumsCountResponse(
            **await self.api.request("photos.getAlbumsCount", params)
        ).response

    async def get_all(
        self,
        owner_id: Optional[int] = None,
        extended: Optional[bool] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        photo_sizes: Optional[bool] = None,
        no_service_albums: Optional[bool] = None,
        need_hidden: Optional[bool] = None,
        skip_hidden: Optional[bool] = None,
        **kwargs
    ) -> photos.GetAllExtendedResponseModel:
        """Returns a list of photos belonging to a user or community, in reverse chronological order.
        :param owner_id: ID of a user or community that owns the photos. Use a negative value to designate a community ID.
        :param extended: '1' — to return detailed information about photos
        :param offset: Offset needed to return a specific subset of photos. By default, '0'.
        :param count: Number of photos to return.
        :param photo_sizes: '1' – to return image sizes in [vk.com/dev/photo_sizes|special format].
        :param no_service_albums: '1' – to return photos only from standard albums, '0' – to return all photos including those in service albums, e.g., 'My wall photos' (default)
        :param need_hidden: '1' – to show information about photos being hidden from the block above the wall.
        :param skip_hidden: '1' – not to return photos being hidden from the block above the wall. Works only with owner_id>0, no_service_albums is ignored.
        """

        params = self.get_set_params(locals())
        return photos.GetAllExtendedResponse(
            **await self.api.request("photos.getAll", params)
        ).response

    async def get_all_comments(
        self,
        owner_id: Optional[int] = None,
        album_id: Optional[int] = None,
        need_likes: Optional[bool] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        **kwargs
    ) -> photos.GetAllCommentsResponseModel:
        """Returns a list of comments on a specific photo album or all albums of the user sorted in reverse chronological order.
        :param owner_id: ID of the user or community that owns the album(s).
        :param album_id: Album ID. If the parameter is not set, comments on all of the user's albums will be returned.
        :param need_likes: '1' — to return an additional 'likes' field, '0' — (default)
        :param offset: Offset needed to return a specific subset of comments. By default, '0'.
        :param count: Number of comments to return. By default, '20'. Maximum value, '100'.
        """

        params = self.get_set_params(locals())
        return photos.GetAllCommentsResponse(
            **await self.api.request("photos.getAllComments", params)
        ).response

    async def get_by_id(
        self,
        photos_: List[str],
        extended: Optional[bool] = None,
        photo_sizes: Optional[bool] = None,
        **kwargs
    ) -> photos.GetByIdExtendedResponseModel:
        """Returns information about photos by their IDs.
        :param photos_: IDs separated with a comma, that are IDs of users who posted photos and IDs of photos themselves with an underscore character between such IDs. To get information about a photo in the group album, you shall specify group ID instead of user ID. Example: "1_129207899,6492_135055734, , -20629724_271945303"
        :param extended: '1' — to return additional fields, '0' — (default)
        :param photo_sizes: '1' — to return photo sizes in a
        """

        params = self.get_set_params(locals())
        return photos.GetByIdExtendedResponse(
            **await self.api.request("photos.getById", params)
        ).response

    async def get_chat_upload_server(
        self,
        chat_id: int,
        crop_x: Optional[int] = None,
        crop_y: Optional[int] = None,
        crop_width: Optional[int] = None,
        **kwargs
    ) -> base.GetUploadServerResponseModel:
        """Returns an upload link for chat cover pictures.
        :param chat_id: ID of the chat for which you want to upload a cover photo.
        :param crop_x:
        :param crop_y:
        :param crop_width: Width (in pixels, **kwargs) of the photo after cropping.
        """

        params = self.get_set_params(locals())
        return base.GetUploadServerResponse(
            **await self.api.request("photos.getChatUploadServer", params)
        ).response

    async def get_comments(
        self,
        photo_id: int,
        owner_id: Optional[int] = None,
        need_likes: Optional[bool] = None,
        start_comment_id: Optional[int] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        sort: Optional[str] = None,
        access_key: Optional[str] = None,
        extended: Optional[bool] = None,
        fields: Optional[List["objects_users.Fields"]] = None,
        **kwargs
    ) -> photos.GetCommentsExtendedResponseModel:
        """Returns a list of comments on a photo.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param need_likes: '1' — to return an additional 'likes' field, '0' — (default)
        :param start_comment_id:
        :param offset: Offset needed to return a specific subset of comments. By default, '0'.
        :param count: Number of comments to return.
        :param sort: Sort order: 'asc' — old first, 'desc' — new first
        :param access_key:
        :param extended:
        :param fields:
        """

        params = self.get_set_params(locals())
        return photos.GetCommentsExtendedResponse(
            **await self.api.request("photos.getComments", params)
        ).response

    async def get_market_album_upload_server(
        self, group_id: int, **kwargs
    ) -> base.GetUploadServerResponseModel:
        """Returns the server address for market album photo upload.
        :param group_id: Community ID.
        """

        params = self.get_set_params(locals())
        return base.GetUploadServerResponse(
            **await self.api.request("photos.getMarketAlbumUploadServer", params)
        ).response

    async def get_market_upload_server(
        self,
        group_id: int,
        main_photo: Optional[bool] = None,
        crop_x: Optional[int] = None,
        crop_y: Optional[int] = None,
        crop_width: Optional[int] = None,
        **kwargs
    ) -> photos.GetMarketUploadServerResponseModel:
        """Returns the server address for market photo upload.
        :param group_id: Community ID.
        :param main_photo: '1' if you want to upload the main item photo.
        :param crop_x: X coordinate of the crop left upper corner.
        :param crop_y: Y coordinate of the crop left upper corner.
        :param crop_width: Width of the cropped photo in px.
        """

        params = self.get_set_params(locals())
        return photos.GetMarketUploadServerResponse(
            **await self.api.request("photos.getMarketUploadServer", params)
        ).response

    async def get_messages_upload_server(
        self, peer_id: Optional[int] = None, **kwargs
    ) -> photos.GetMessagesUploadServerResponseModel:
        """Returns the server address for photo upload in a private message for a user.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        """

        params = self.get_set_params(locals())
        return photos.GetMessagesUploadServerResponse(
            **await self.api.request("photos.getMessagesUploadServer", params)
        ).response

    async def get_new_tags(
        self, offset: Optional[int] = None, count: Optional[int] = None, **kwargs
    ) -> photos.GetNewTagsResponseModel:
        """Returns a list of photos with tags that have not been viewed.
        :param offset: Offset needed to return a specific subset of photos.
        :param count: Number of photos to return.
        """

        params = self.get_set_params(locals())
        return photos.GetNewTagsResponse(
            **await self.api.request("photos.getNewTags", params)
        ).response

    async def get_owner_cover_photo_upload_server(
        self,
        group_id: int,
        crop_x: Optional[int] = None,
        crop_y: Optional[int] = None,
        crop_x2: Optional[int] = None,
        crop_y2: Optional[int] = None,
        **kwargs
    ) -> base.GetUploadServerResponseModel:
        """Returns the server address for owner cover upload.
        :param group_id: ID of community that owns the album (if the photo will be uploaded to a community album).
        :param crop_x: X coordinate of the left-upper corner
        :param crop_y: Y coordinate of the left-upper corner
        :param crop_x2: X coordinate of the right-bottom corner
        :param crop_y2: Y coordinate of the right-bottom corner
        """

        params = self.get_set_params(locals())
        return base.GetUploadServerResponse(
            **await self.api.request("photos.getOwnerCoverPhotoUploadServer", params)
        ).response

    async def get_owner_photo_upload_server(
        self, owner_id: Optional[int] = None, **kwargs
    ) -> base.GetUploadServerResponseModel:
        """Returns an upload server address for a profile or community photo.
        :param owner_id: identifier of a community or current user. "Note that community id must be negative. 'owner_id=1' – user, 'owner_id=-1' – community, "
        """

        params = self.get_set_params(locals())
        return base.GetUploadServerResponse(
            **await self.api.request("photos.getOwnerPhotoUploadServer", params)
        ).response

    async def get_tags(
        self,
        photo_id: int,
        owner_id: Optional[int] = None,
        access_key: Optional[str] = None,
        **kwargs
    ) -> photos.GetTagsResponseModel:
        """Returns a list of tags on a photo.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param access_key:
        """

        params = self.get_set_params(locals())
        return photos.GetTagsResponse(
            **await self.api.request("photos.getTags", params)
        ).response

    async def get_upload_server(
        self, group_id: Optional[int] = None, album_id: Optional[int] = None, **kwargs
    ) -> photos.GetUploadServerResponseModel:
        """Returns the server address for photo upload.
        :param group_id: ID of community that owns the album (if the photo will be uploaded to a community album).
        :param album_id:
        """

        params = self.get_set_params(locals())
        return photos.GetUploadServerResponse(
            **await self.api.request("photos.getUploadServer", params)
        ).response

    async def get_user_photos(
        self,
        user_id: Optional[int] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        extended: Optional[bool] = None,
        sort: Optional[str] = None,
        **kwargs
    ) -> photos.GetUserPhotosExtendedResponseModel:
        """Returns a list of photos in which a user is tagged.
        :param user_id: User ID.
        :param offset: Offset needed to return a specific subset of photos. By default, '0'.
        :param count: Number of photos to return. Maximum value is 1000.
        :param extended: '1' — to return an additional 'likes' field, '0' — (default)
        :param sort: Sort order: '1' — by date the tag was added in ascending order, '0' — by date the tag was added in descending order
        """

        params = self.get_set_params(locals())
        return photos.GetUserPhotosExtendedResponse(
            **await self.api.request("photos.getUserPhotos", params)
        ).response

    async def get_wall_upload_server(
        self, group_id: Optional[int] = None, **kwargs
    ) -> photos.GetWallUploadServerResponseModel:
        """Returns the server address for photo upload onto a user's wall.
        :param group_id: ID of community to whose wall the photo will be uploaded.
        """

        params = self.get_set_params(locals())
        return photos.GetWallUploadServerResponse(
            **await self.api.request("photos.getWallUploadServer", params)
        ).response

    async def make_cover(
        self,
        photo_id: int,
        owner_id: Optional[int] = None,
        album_id: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Makes a photo into an album cover.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param album_id: Album ID.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.makeCover", params)
        ).response

    async def move(
        self,
        target_album_id: int,
        photo_id: int,
        owner_id: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Moves a photo from one album to another.
        :param target_album_id: ID of the album to which the photo will be moved.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(**await self.api.request("photos.move", params)).response

    async def put_tag(
        self,
        photo_id: int,
        user_id: int,
        owner_id: Optional[int] = None,
        x: Optional[float] = None,
        y: Optional[float] = None,
        x2: Optional[float] = None,
        y2: Optional[float] = None,
        **kwargs
    ) -> photos.PutTagResponseModel:
        """Adds a tag on the photo.
        :param photo_id: Photo ID.
        :param user_id: ID of the user to be tagged.
        :param owner_id: ID of the user or community that owns the photo.
        :param x: Upper left-corner coordinate of the tagged area (as a percentage of the photo's width).
        :param y: Upper left-corner coordinate of the tagged area (as a percentage of the photo's height).
        :param x2: Lower right-corner coordinate of the tagged area (as a percentage of the photo's width).
        :param y2: Lower right-corner coordinate of the tagged area (as a percentage of the photo's height).
        """

        params = self.get_set_params(locals())
        return photos.PutTagResponse(
            **await self.api.request("photos.putTag", params)
        ).response

    async def remove_tag(
        self, photo_id: int, tag_id: int, owner_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Removes a tag from a photo.
        :param photo_id: Photo ID.
        :param tag_id: Tag ID.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.removeTag", params)
        ).response

    async def reorder_albums(
        self,
        album_id: int,
        owner_id: Optional[int] = None,
        before: Optional[int] = None,
        after: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Reorders the album in the list of user albums.
        :param album_id: Album ID.
        :param owner_id: ID of the user or community that owns the album.
        :param before: ID of the album before which the album in question shall be placed.
        :param after: ID of the album after which the album in question shall be placed.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.reorderAlbums", params)
        ).response

    async def reorder_photos(
        self,
        photo_id: int,
        owner_id: Optional[int] = None,
        before: Optional[int] = None,
        after: Optional[int] = None,
        **kwargs
    ) -> base.OkResponseModel:
        """Reorders the photo in the list of photos of the user album.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param before: ID of the photo before which the photo in question shall be placed.
        :param after: ID of the photo after which the photo in question shall be placed.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.reorderPhotos", params)
        ).response

    async def report(
        self, owner_id: int, photo_id: int, reason: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Reports (submits a complaint about, **kwargs) a photo.
        :param owner_id: ID of the user or community that owns the photo.
        :param photo_id: Photo ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.report", params)
        ).response

    async def report_comment(
        self, owner_id: int, comment_id: int, reason: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Reports (submits a complaint about, **kwargs) a comment on a photo.
        :param owner_id: ID of the user or community that owns the photo.
        :param comment_id: ID of the comment being reported.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.reportComment", params)
        ).response

    async def restore(
        self, photo_id: int, owner_id: Optional[int] = None, **kwargs
    ) -> base.OkResponseModel:
        """Restores a deleted photo.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return base.OkResponse(
            **await self.api.request("photos.restore", params)
        ).response

    async def restore_comment(
        self, comment_id: int, owner_id: Optional[int] = None, **kwargs
    ) -> photos.RestoreCommentResponseModel:
        """Restores a deleted comment on a photo.
        :param comment_id: ID of the deleted comment.
        :param owner_id: ID of the user or community that owns the photo.
        """

        params = self.get_set_params(locals())
        return photos.RestoreCommentResponse(
            **await self.api.request("photos.restoreComment", params)
        ).response

    async def save(
        self,
        album_id: Optional[int] = None,
        group_id: Optional[int] = None,
        server: Optional[int] = None,
        photos_list: Optional[str] = None,
        hash: Optional[str] = None,
        latitude: Optional[float] = None,
        longitude: Optional[float] = None,
        caption: Optional[str] = None,
        **kwargs
    ) -> photos.SaveResponseModel:
        """Saves photos after successful uploading.
        :param album_id: ID of the album to save photos to.
        :param group_id: ID of the community to save photos to.
        :param server: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param photos_list: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param hash: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param latitude: Geographical latitude, in degrees (from '-90' to '90').
        :param longitude: Geographical longitude, in degrees (from '-180' to '180').
        :param caption: Text describing the photo. 2048 digits max.
        """

        params = self.get_set_params(locals())
        return photos.SaveResponse(
            **await self.api.request("photos.save", params)
        ).response

    async def save_market_album_photo(
        self, group_id: int, photo: str, server: int, hash: str, **kwargs
    ) -> photos.SaveMarketAlbumPhotoResponseModel:
        """Saves market album photos after successful uploading.
        :param group_id: Community ID.
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param server: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param hash: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """

        params = self.get_set_params(locals())
        return photos.SaveMarketAlbumPhotoResponse(
            **await self.api.request("photos.saveMarketAlbumPhoto", params)
        ).response

    async def save_market_photo(
        self,
        photo: str,
        server: int,
        hash: str,
        group_id: Optional[int] = None,
        crop_data: Optional[str] = None,
        crop_hash: Optional[str] = None,
        **kwargs
    ) -> photos.SaveMarketPhotoResponseModel:
        """Saves market photos after successful uploading.
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param server: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param hash: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param group_id: Community ID.
        :param crop_data: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param crop_hash: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """

        params = self.get_set_params(locals())
        return photos.SaveMarketPhotoResponse(
            **await self.api.request("photos.saveMarketPhoto", params)
        ).response

    async def save_messages_photo(
        self,
        photo: str,
        server: Optional[int] = None,
        hash: Optional[str] = None,
        **kwargs
    ) -> photos.SaveMessagesPhotoResponseModel:
        """Saves a photo after being successfully uploaded. URL obtained with [vk.com/dev/photos.getMessagesUploadServer|photos.getMessagesUploadServer] method.
        :param photo: Parameter returned when the photo is [vk.com/dev/upload_files|uploaded to the server].
        :param server:
        :param hash:
        """

        params = self.get_set_params(locals())
        return photos.SaveMessagesPhotoResponse(
            **await self.api.request("photos.saveMessagesPhoto", params)
        ).response

    async def save_owner_cover_photo(
        self, hash: str, photo: str, **kwargs
    ) -> photos.SaveOwnerCoverPhotoResponseModel:
        """Saves cover photo after successful uploading.
        :param hash: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """

        params = self.get_set_params(locals())
        return photos.SaveOwnerCoverPhotoResponse(
            **await self.api.request("photos.saveOwnerCoverPhoto", params)
        ).response

    async def save_owner_photo(
        self,
        server: Optional[str] = None,
        hash: Optional[str] = None,
        photo: Optional[str] = None,
        **kwargs
    ) -> photos.SaveOwnerPhotoResponseModel:
        """Saves a profile or community photo. Upload URL can be got with the [vk.com/dev/photos.getOwnerPhotoUploadServer|photos.getOwnerPhotoUploadServer] method.
        :param server: parameter returned after [vk.com/dev/upload_files|photo upload].
        :param hash: parameter returned after [vk.com/dev/upload_files|photo upload].
        :param photo: parameter returned after [vk.com/dev/upload_files|photo upload].
        """

        params = self.get_set_params(locals())
        return photos.SaveOwnerPhotoResponse(
            **await self.api.request("photos.saveOwnerPhoto", params)
        ).response

    async def save_wall_photo(
        self,
        photo: str,
        user_id: Optional[int] = None,
        group_id: Optional[int] = None,
        server: Optional[int] = None,
        hash: Optional[str] = None,
        latitude: Optional[float] = None,
        longitude: Optional[float] = None,
        caption: Optional[str] = None,
        **kwargs
    ) -> photos.SaveWallPhotoResponseModel:
        """Saves a photo to a user's or community's wall after being uploaded.
        :param photo: Parameter returned when the the photo is [vk.com/dev/upload_files|uploaded to the server].
        :param user_id: ID of the user on whose wall the photo will be saved.
        :param group_id: ID of community on whose wall the photo will be saved.
        :param server:
        :param hash:
        :param latitude: Geographical latitude, in degrees (from '-90' to '90').
        :param longitude: Geographical longitude, in degrees (from '-180' to '180').
        :param caption: Text describing the photo. 2048 digits max.
        """

        params = self.get_set_params(locals())
        return photos.SaveWallPhotoResponse(
            **await self.api.request("photos.saveWallPhoto", params)
        ).response

    async def search(
        self,
        q: Optional[str] = None,
        lat: Optional[float] = None,
        long: Optional[float] = None,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        sort: Optional[int] = None,
        offset: Optional[int] = None,
        count: Optional[int] = None,
        radius: Optional[int] = None,
        **kwargs
    ) -> photos.SearchResponseModel:
        """Returns a list of photos.
        :param q: Search query string.
        :param lat: Geographical latitude, in degrees (from '-90' to '90').
        :param long: Geographical longitude, in degrees (from '-180' to '180').
        :param start_time:
        :param end_time:
        :param sort: Sort order:
        :param offset: Offset needed to return a specific subset of photos.
        :param count: Number of photos to return.
        :param radius: Radius of search in meters (works very approximately). Available values: '10', '100', '800', '6000', '50000'.
        """

        params = self.get_set_params(locals())
        return photos.SearchResponse(
            **await self.api.request("photos.search", params)
        ).response



from .base_event_object import BaseEventObject
from vkbottle_types.objects import (
    WallWallComment,
    WallWallpost,
    BoardTopicComment,
    PhotosPhoto,
    CallbackGroupJoinType,
    UsersBlockReason,
    GroupsGroupAdminLevel,
    MessagesMessage,
    MessagesClientInfo,
    AudioAudio,
    ObjectType,
    VideoVideo,
    MarketOrder,
)
import enum
from typing import Optional, Any


class MessageNewObject(BaseEventObject):
    message: Optional[MessagesMessage] = None
    client_info: Optional[MessagesClientInfo] = None


class MessageObject(MessagesMessage):
    pass


class MessageAllowObject(BaseEventObject):
    user_id: Optional[int] = None
    key: Optional[str] = None
    api: Optional[list] = None


class MessageTypingStateObject(BaseEventObject):
    state: Optional[str] = None
    from_id: Optional[int] = None
    to_id: Optional[int] = None
    api: Optional[list] = None


class MessageDenyObject(BaseEventObject):
    user_id: Optional[int] = None


class MessageEventObject(BaseEventObject):
    user_id: Optional[int] = None
    peer_id: Optional[int] = None
    event_id: Optional[str] = None
    payload: Optional[str] = None
    conversation_message_id: Optional[int] = None


class PhotoNewObject(PhotosPhoto):
    pass


class PhotoCommentObject(WallWallComment):
    photo_id: Optional[int] = None
    photo_owner_id: Optional[int] = None


class PhotoCommentDeleteObject(BaseEventObject):
    owner_id: Optional[int] = None
    id: Optional[int] = None
    user_id: Optional[int] = None
    photo_id: Optional[int] = None


class AudioNewObject(AudioAudio):
    pass


class VideoNewObject(VideoVideo):
    pass


class VideoCommentObject(WallWallComment):
    video_id: Optional[int] = None
    video_owner_id: Optional[int] = None


class VideoCommentDeleteObject(BaseEventObject):
    owner_id: Optional[int] = None
    id: Optional[int] = None
    user_id: Optional[int] = None
    video_id: Optional[int] = None


class WallPostNewObject(WallWallpost):
    postponed_id: Optional[int] = None


class WallReplyNewObject(WallWallComment):
    post_id: Optional[int] = None
    post_owner_id: Optional[int] = None
    api: Optional[list] = None


class WallReplyDeleteObject(BaseEventObject):
    owner_id: Optional[int] = None
    id: Optional[int] = None
    user_id: Optional[int] = None
    deleter_id: Optional[int] = None
    post_id: Optional[int] = None


class LikeObject(BaseEventObject):
    liker_id: Optional[int] = None
    object_type: Optional[ObjectType] = None
    object_owner_id: Optional[int] = None
    object_id: Optional[int] = None
    thread_reply_id: Optional[int] = None
    post_id: Optional[int] = None


class BoardPostNewObject(BoardTopicComment):
    topic_id: Optional[int] = None
    topic_owner_id: Optional[int] = None


class BoardPostDeleteObject(BaseEventObject):
    topic_id: Optional[int] = None
    id: Optional[int] = None


class MarketCommentNewObject(WallWallComment):
    market_owner_id: Optional[int] = None
    item_id: Optional[int] = None


class MarketCommentDeleteObject(BaseEventObject):
    owner_id: Optional[int] = None
    id: Optional[int] = None
    user_id: Optional[int] = None
    item_id: Optional[int] = None


class GroupLeaveObject(BaseEventObject):
    user_id: Optional[int] = None
    self: Optional[int] = None


class GroupJoinObject(BaseEventObject):
    user_id: Optional[int] = None
    join_type: Optional[CallbackGroupJoinType] = None


class UserBlockObject(BaseEventObject):
    admin_id: Optional[int] = None
    user_id: Optional[int] = None
    unblock_data: Optional[int] = None
    reason: Optional[UsersBlockReason] = None
    comment: Optional[str] = None


class UserUnblockObject(BaseEventObject):
    admin_id: Optional[int] = None
    user_id: Optional[int] = None
    by_end_date: Optional[int] = None


class PollVoteNewObject(BaseEventObject):
    owner_id: Optional[int] = None
    poll_id: Optional[int] = None
    option_id: Optional[int] = None
    user_id: Optional[int] = None


class GroupOfficersEditObject(BaseEventObject):
    admin_id: Optional[int] = None
    user_id: Optional[int] = None
    level_old: Optional[GroupsGroupAdminLevel] = None
    level_new: Optional[GroupsGroupAdminLevel] = None


class GroupChangeSettingsChangesSectionEnableObject(enum.Enum):
    STATUS_DEFAULT = "status_default"
    AUDIO = "audio"
    PHOTO = "photo"
    VIDEO = "video"
    MARKET = "market"


class GroupChangeSettingsChangesSectionNameObject(enum.Enum):
    TITLE = "title"
    DESCRIPTION = "description"
    COMMUNITY_TYPE = "access"
    SCREEN_NAME = "screen_name"
    PUBLIC_CATEGORY = "public_category"
    PUBLIC_SUBCATEGORY = "public_subcategory"
    AGE_LIMITS = "age_limits"
    WEBSITE = "website"
    ENABLE_SECTION = GroupChangeSettingsChangesSectionEnableObject


class GroupChangeSettingsChangesObject(BaseEventObject):
    section_name: Optional[GroupChangeSettingsChangesSectionNameObject] = None
    old_value: Any = None
    new_value: Any = None


class GroupChangeSettingsObject(BaseEventObject):
    user_id: Optional[int] = None
    changes: Optional[GroupChangeSettingsChangesObject] = None


class GroupChangePhotoObject(BaseEventObject):
    user_id: Optional[int] = None
    photo: Optional[PhotosPhoto] = None


class MarketOrderObject(MarketOrder):
    pass


class VkPayTransactionObject(BaseEventObject):
    from_id: Optional[int] = None
    amount: Optional[int] = None
    description: Optional[str] = None
    date: Optional[int] = None


class AppPayloadObject(BaseEventObject):
    user_id: Optional[int] = None
    app_id: Optional[int] = None
    payload: Optional[str] = None
    group_id: Optional[int] = None


MessageAllowObject.update_forward_refs()
MessageTypingStateObject.update_forward_refs()
MessageDenyObject.update_forward_refs()
PhotoCommentObject.update_forward_refs()
PhotoCommentDeleteObject.update_forward_refs()
VideoCommentObject.update_forward_refs()
VideoCommentDeleteObject.update_forward_refs()
WallPostNewObject.update_forward_refs()
WallReplyNewObject.update_forward_refs()
WallReplyDeleteObject.update_forward_refs()
BoardPostNewObject.update_forward_refs()
BoardPostDeleteObject.update_forward_refs()
MarketCommentNewObject.update_forward_refs()
MarketCommentDeleteObject.update_forward_refs()
GroupLeaveObject.update_forward_refs()
GroupJoinObject.update_forward_refs()
UserBlockObject.update_forward_refs()
UserUnblockObject.update_forward_refs()
PollVoteNewObject.update_forward_refs()
GroupOfficersEditObject.update_forward_refs()
GroupChangeSettingsChangesObject.update_forward_refs()
GroupChangeSettingsObject.update_forward_refs()
GroupChangePhotoObject.update_forward_refs()



from typing import Union, Dict, Type, Callable, Any, TYPE_CHECKING
from .enums import GroupEventType, UserEventType


if TYPE_CHECKING:
    from .typings import GroupTypes, UserTypes
    from .bot_events import BaseGroupEvent
    from .user_events import BaseUserEvent


class EventsBase:
    events_registered: Dict[
        Union[GroupEventType, UserEventType],
        Type[Union["GroupTypes.UnifiedTypes", "UserTypes.UnifiedTypes"]],
    ] = {}

    def __init__(
        self,
        type_enum: Type[Union[GroupEventType, UserEventType]],
        on_undefined: Callable[[str], Any] = lambda et: exec(
            f"raise NotImplementedError({et})"
        ),
    ):
        self.type_enum = type_enum
        self.on_undefined = on_undefined

    def register(
        self,
        event_type: Union[GroupEventType, UserEventType],
        event_model: Union["BaseGroupEvent", "BaseUserEvent"],
    ):
        self.events_registered[event_type] = event_model

    def get(
        self, event_type: Union[int, str, GroupEventType, UserEventType]
    ) -> Callable[..., Union["GroupTypes.UnifiedTypes", "UserTypes.UnifiedTypes"]]:
        if not str(event_type).startswith(self.type_enum.__name__):
            event_type = self.type_enum(event_type)

        model = self.events_registered.get(event_type)
        if model is None:
            return self.on_undefined(event_type)
        return model



import enum


class UserEventType(enum.IntEnum):
    REPLACE_MESSAGE_FLAGS = 1
    INSTALL_MESSAGE_FLAGS = 2
    RESET_MESSAGE_FLAGS = 3

    NEW_MESSAGE = 4
    EDIT_MESSAGE = 5
    IN_READ = 6
    OUT_READ = 7
    FRIEND_ONLINE = 8
    FRIEND_OFFLINE = 9
    RESET_DIALOG_FLAGS = 10
    REPLACE_DIALOG_FLAGS = 11
    INSTALL_DIALOG_FLAGS = 12
    DELETE_MESSAGES = 13
    CHAT_RESTORE = 14

    CHANGE_MAJOR_ID = 20
    CHANGE_MINOR_ID = 21

    TOPIC_PARAMS_CHANGE = 51
    CHAT_INFO_EDIT = 52
    DIALOG_TYPING_STATE = 61

    USER_TYPING_STATE = 62
    USERS_TYPING_STATE = 63
    VOICE_MESSAGE_STATES = 64

    CALL = 70
    COUNTER = 80
    NOTIFICATIONS_SETTINGS_CHANGED = 114



from .bot_events import *
from .user_events import *
from typing import Any, Union


class GroupTypes:
    MessageNew = MessageNew
    MessageAllow = MessageAllow
    MessageReply = Message
    MessageEdit = Message
    MessageTypingState = MessageTypingState
    MessageDeny = MessageDeny
    MessageEvent = MessageEvent
    PhotoNew = PhotoNew
    PhotoComment = PhotoComment
    PhotoCommentDelete = PhotoCommentDelete
    AudioNew = AudioNew
    VideoNew = VideoNew
    VideoCommentNew = VideoComment
    VideoCommentEdit = VideoComment
    VideoCommentRestore = VideoComment
    VideoCommentDelete = VideoCommentDelete
    WallPostNew = WallPostNew
    WallRepost = WallPostNew
    WallReplyNew = WallReply
    WallReplyEdit = WallReply
    WallReplyRestore = WallReply
    WallReplyDelete = WallReplyDelete
    LikeAdd = Like
    LikeRemove = Like
    BoardPostNew = BoardPost
    BoardPostEdit = BoardPost
    BoardPostRestore = BoardPost
    BoardPostDelete = BoardPostDelete
    MarketCommentNew = MarketComment
    MarketCommentEdit = MarketComment
    MarketCommentRestore = MarketComment
    MarketCommentDelete = MarketCommentDelete
    GroupJoin = GroupJoin
    GroupLeave = GroupLeave
    UserBlock = UserBlock
    UserUnblock = UserUnblock
    PollVoteNew = PollVoteNew
    GroupOfficersEdit = GroupOfficersEdit
    GroupChangeSettings = GroupChangeSettings
    GroupChangePhoto = GroupChangePhoto
    VkPayTransaction = VkPayTransaction
    AppPayload = AppPayload

    UnifiedTypes = Union[
        BaseGroupEvent,
        MessageNew,
        Message,
        MessageAllow,
        MessageTypingState,
        MessageDeny,
        MessageEvent,
        PhotoNew,
        PhotoComment,
        PhotoCommentDelete,
        AudioNew,
        VideoNew,
        VideoComment,
        VideoCommentDelete,
        WallPostNew,
        WallReply,
        WallReplyDelete,
        Like,
        BoardPost,
        BoardPostDelete,
        MarketComment,
        MarketCommentDelete,
        GroupJoin,
        GroupLeave,
        UserBlock,
        UserUnblock,
        PollVoteNew,
        GroupOfficersEdit,
        GroupChangeSettings,
        GroupChangePhoto,
        VkPayTransaction,
        AppPayload,
    ]


class UserTypes:
    UnifiedTypes = Union[BaseUserEvent, Any]
